"""
A collection of useful functions
"""

__all__ = ['add', 'subtract', 'multiply']


def add(a, b):
    """
    Add two numbers
    """
    return a + b


def subtract(a, b):
    """
    Subtract two numbers
    """
    return a - b


def multiply(c, d):
    """
    Multiply two numbers
    """
    return c * d
