__all__ = ['NonAsciiÄöüßő']


class NonAsciiÄöüßő:
    def get_äöü(self):
        """
        Return a string with common umlauts like äöüß
        """
        return 'äöü'

    def get_ß(self):
        """
        Return a string with the eszett symbol
        """
        return 'ß'
