class ClassWithAttribute:
    """A class with an attribute."""
    my_attribute = 1

    def my_method(self):
        """A method."""
        pass

    @property
    def my_property(self):
        """A property."""
        return 1
