"""
A collection of useful classes and functions
"""

__all__ = ['add', 'MixedSpam']


def add(a, b):
    """
    Add two numbers
    """
    return a + b


class MixedSpam:
    """
    Special spam
    """

    def eat(self, time):
        """
        Eat special spam in the required time.
        """
        pass
